@@ -21,32 +21,26 @@
  */
 
 #include <config.h>
-#include <gnome.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <gnome.h>
-#include <libbonobo.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include "actions.h"
-#include "main.h"
-#include "gtk-utils.h"
-#include "goo-window.h"
-#include "file-utils.h"
-#include "gconf-utils.h"
-#include "preferences.h"
 #include "dlg-extract.h"
 #include "dlg-preferences.h"
 #include "dlg-properties.h"
+#include "gtk-utils.h"
+#include "goo-window.h"
+#include "gconf-utils.h"
+#include "main.h"
+#include "preferences.h"
 
 
 void
 activate_action_play (GtkAction *action, 
 		      gpointer   data)
 {
-	GooWindow *window = data;
-	goo_window_play (window);
+	goo_window_play (GOO_WINDOW (data));
 }
 
 
@@ -54,8 +48,7 @@ void
 activate_action_play_selected (GtkAction *action, 
 			       gpointer   data)
 {
-	GooWindow *window = data;
-	goo_window_play_selected (window);
+	goo_window_play_selected (GOO_WINDOW (data));
 }
 
 
@@ -63,8 +56,7 @@ void
 activate_action_pause (GtkAction *action,
 		       gpointer   data)
 {
-	GooWindow *window = data;
-	goo_window_pause (window);
+	goo_window_pause (GOO_WINDOW (data));
 }
 
 
@@ -72,8 +64,7 @@ void
 activate_action_toggle_play (GtkAction *action, 
 			     gpointer   data)
 {
-	GooWindow *window = data;
-	goo_window_toggle_play (window);
+	goo_window_toggle_play (GOO_WINDOW (data));
 }
 
 
@@ -81,8 +72,7 @@ void
 activate_action_stop (GtkAction *action, 
 		      gpointer   data)
 {
-	GooWindow *window = data;
-	goo_window_stop (window);
+	goo_window_stop (GOO_WINDOW (data));
 }
 
 
@@ -90,8 +80,7 @@ void
 activate_action_next (GtkAction *action, 
 		      gpointer   data)
 {
-	GooWindow *window = data;
-	goo_window_next (window);
+	goo_window_next (GOO_WINDOW (data));
 }
 
 
@@ -99,8 +88,7 @@ void
 activate_action_prev (GtkAction *action, 
 		      gpointer   data)
 {
-	GooWindow *window = data;
-	goo_window_prev (window);
+	goo_window_prev (GOO_WINDOW (data));
 }
 
 
@@ -108,8 +96,7 @@ void
 activate_action_eject (GtkAction *action, 
 		       gpointer   data)
 {
-	GooWindow *window = data;
-	goo_window_eject (window);
+	goo_window_eject (GOO_WINDOW (data));
 }
 
 
@@ -117,41 +104,7 @@ void
 activate_action_reload (GtkAction *action, 
 			gpointer   data)
 {
-	GooWindow *window = data;
-	goo_window_update (window);
-}
-
-
-
-static void
-show_help (GooWindow  *window,
-	   const char *section)
-{
-	GError *err = NULL;  
-
-        gnome_help_display ("goobox", section, &err);
-        
-        if (err != NULL) {
-                GtkWidget *dialog;
-                
-                dialog = _gtk_message_dialog_new (GTK_WINDOW (window),
-						  GTK_DIALOG_DESTROY_WITH_PARENT, 
-						  GTK_STOCK_DIALOG_ERROR,
-						  _("Could not display help"),
-						  err->message,
-						  GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
-						  NULL);
-                gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
-                g_signal_connect (G_OBJECT (dialog), "response",
-                                  G_CALLBACK (gtk_widget_destroy),
-                                  NULL);
-                
-                gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-                
-                gtk_widget_show (dialog);
-                
-                g_error_free (err);
-        }
+	goo_window_update (GOO_WINDOW (data));
 }
 
 
@@ -159,8 +112,7 @@ void
 activate_action_manual (GtkAction *action, 
 			gpointer   data)
 {
-	GooWindow *window = data;
-	show_help (window, NULL);
+	show_help_dialog (GTK_WINDOW (data), NULL);
 }
 
 
@@ -168,8 +120,7 @@ void
 activate_action_shortcuts (GtkAction *action, 
 			   gpointer   data)
 {
-	GooWindow *window = data;
-	show_help (window, "goobox-shortcuts");
+	show_help_dialog (GTK_WINDOW (data), "goobox-shortcuts");
 }
 
 
@@ -179,27 +130,27 @@ activate_action_about (GtkAction *action,
 {
 	GooWindow  *window = data;
 	const char *authors[] = {
-		"Paolo Bacchilega <paolo.bacchilega@libero.it>", 
+		"Paolo Bacchilega <paobac@src.gnome.org>",
+		NULL
+	};
+	const char *documenters [] = {
+		"Paolo Bacchilega <paobac@src.gnome.org>",
 		NULL
 	};
-	/*
-	  const char     *documenters [] = {
-	  NULL
-	  };*/
 	const char *translator_credits = _("translator_credits");
         char       *license_text;
         const char *license[] = {
-                "CD Player is free software; you can redistribute it and/or modify"
+                N_("CD Player is free software; you can redistribute it and/or modify"
                 "it under the terms of the GNU General Public License as published by "
                 "the Free Software Foundation; either version 2 of the License, or "
-                "(at your option) any later version.",
-                "CD Player is distributed in the hope that it will be useful, "
+                "(at your option) any later version."),
+                N_("CD Player is distributed in the hope that it will be useful, "
                 "but WITHOUT ANY WARRANTY; without even the implied warranty of "
                 "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
-                "GNU General Public License for more details.",
-                "You should have received a copy of the GNU General Public License "
+                "GNU General Public License for more details."),
+                N_("You should have received a copy of the GNU General Public License "
                 "along with CD Player; if not, write to the Free Software Foundation, Inc., "
-                "51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA"
+                "51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA")
         };
 
         license_text = g_strconcat (license[0], "\n\n", license[1], "\n\n",
@@ -208,10 +159,10 @@ activate_action_about (GtkAction *action,
         gtk_show_about_dialog (GTK_WINDOW (window),
         		       "name", _("CD Player"),
                                "version", VERSION,
-                               "copyright", _("Copyright \xc2\xa9 2004-2007 Free Software Foundation, Inc."),
+                               "copyright", _("Copyright \xc2\xa9 2004-2009 Free Software Foundation, Inc."),
                                "comments", _("Play CDs and save the tracks to disk as files"),
                                "authors", authors,
-                               /*"documenters", documenters,*/
+                               "documenters", documenters,
                                "translator_credits", strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
                                "logo-icon-name", "goobox",
                                "license", license_text,
@@ -271,21 +222,20 @@ activate_action_shuffle (GtkAction *action,
 
 
 static void
-sj_watch_func (GPid     pid,
-	       gint     status,
-	       gpointer data)
+external_app_watch_func (GPid     pid,
+	       	         gint     status,
+	                 gpointer data)
 {
-	GooWindow *window = data;
-	
 	g_spawn_close_pid (pid);
-	goo_window_set_hibernate (window, FALSE);
+	goo_window_set_hibernate (GOO_WINDOW (data), FALSE);
 }
 
 
 static gboolean
-exec_sj (const char  *command_line,
-	 GError     **error,
-	 gpointer     data)
+exec_external_app (GdkScreen   *screen,
+		   const char  *command_line,
+	 	   GError     **error,
+	 	   gpointer     data)
 {
 	gboolean   retval;
 	gchar    **argv = NULL;
@@ -293,47 +243,77 @@ exec_sj (const char  *command_line,
 
 	g_return_val_if_fail (command_line != NULL, FALSE);
 	
-	if (!g_shell_parse_argv (command_line,
-				 NULL, &argv,
-				 error))
+	if (! g_shell_parse_argv (command_line, NULL, &argv, error))
 		return FALSE;
 
-	retval = g_spawn_async (NULL,
-				argv,
-				NULL,
-				G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
-				NULL,
-				NULL,
-				&child_pid,
-				error);
+	retval = gdk_spawn_on_screen (screen,
+				      NULL,
+				      argv,
+				      NULL,
+				      G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
+				      NULL,
+				      NULL,
+				      &child_pid,
+				      error);
 	g_strfreev (argv);
-
-	g_child_watch_add (child_pid,
-			   sj_watch_func,
-			   data);
+	g_child_watch_add (child_pid, external_app_watch_func, data);
 
 	return retval;
 }
 
 
+void
+activate_action_copy_disc (GtkAction *action, 
+			   gpointer   data)
+{
+	GooWindow *window = data;
+	char      *command;
+	GError    *error = NULL;
+
+	command = g_strconcat ("brasero --copy=",
+			       goo_player_get_device (goo_window_get_player (window)), 
+			       NULL);
+
+	goo_window_set_hibernate (window, TRUE);
+	if (! exec_external_app (gtk_widget_get_screen (GTK_WIDGET (window)), 
+				 command, 
+				 &error, 
+				 data)) 
+	{
+		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (window), 
+						   _("Could not execute command"), 
+						   &error);
+		goo_window_set_hibernate (window, FALSE);
+	}
+
+	g_free (command);
+}
+
+
 void
 activate_action_extract (GtkAction *action, 
 			 gpointer   data)
 {
 	GooWindow *window = data;
-
-	if (preferences_get_use_sound_juicer ()) {
-		GError *error = NULL;
-
-		goo_window_set_hibernate (window, TRUE);		
-
-		if (! exec_sj ("sound-juicer", &error, data))
-			_gtk_error_dialog_from_gerror_run (GTK_WINDOW (window), 
-							   _("Could not execute command"), 
-							   &error);
-	}
-	else
+	GError    *error = NULL;
+	
+	if (! preferences_get_use_sound_juicer ()) {
 		dlg_extract (window);
+		return;
+	}
+
+	goo_window_set_hibernate (window, TRUE);		
+
+	if (! exec_external_app (gtk_widget_get_screen (GTK_WIDGET (window)), 
+				 "sound-juicer", 
+				 &error, 
+				 data)) 
+	{
+		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (window), 
+						   _("Could not execute command"), 
+						   &error);
+		goo_window_set_hibernate (window, FALSE);
+	}
 }
 
 
@@ -342,19 +322,25 @@ activate_action_extract_selected (GtkAction *action,
 			 	  gpointer   data)
 {
 	GooWindow *window = data;
+	GError    *error = NULL;
 	
-	if (preferences_get_use_sound_juicer ()) {
-		GError *error = NULL;
-
-		goo_window_set_hibernate (window, TRUE);		
-
-		if (! exec_sj ("sound-juicer", &error, data))
-			_gtk_error_dialog_from_gerror_run (GTK_WINDOW (window), 
-							   _("Could not execute command"), 
-							   &error);
-	}
-	else
+	if (! preferences_get_use_sound_juicer ()) {
 		dlg_extract_selected (window);	
+		return;
+	}
+		
+	goo_window_set_hibernate (window, TRUE);		
+
+	if (! exec_external_app (gtk_widget_get_screen (GTK_WIDGET (window)),
+				 "sound-juicer", 
+				 &error, 
+				 data)) 
+	{
+		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (window), 
+						   _("Could not execute command"), 
+						   &error);
+		goo_window_set_hibernate (window, FALSE);
+	}
 }
 
 
